function [L,R] = LRfact(A,r)

% Implements LR factorization algorithm from Theorem 5.1.
 
% Check size of A
[Nr,Nc] = size(A);
if Nr == Nc
    N = Nr;
else
    disp('Matrix is not square')
    return
end

% Allocate variable for upper triangular matrix R
R = zeros(N);

% Allocate variable for matrices L_k. 
% For k=1:N-r the size of L_k is (r+1)x(r+1), 
% whereas L_{N-r+1} is rxr
% and it is saved in the upper left
% rxr block of L(:,:,N-r+1).
L = zeros(r+1,r+1,N-r+1);

% Run algorithm
%
% Step 1
gamma = A(1,1);
f = A(2:r+1,1)/gamma;
X = A(1,2:N);
Y = -f*X + A(2:r+1,2:N);
L(:,:,1) = [1, zeros(1,r); -f, eye(r)];
R(1,1) = gamma;
R(1,2:N) = A(1,2:N);

% Step 2 (here L_k has size (r+1)x(r+1))
for k = 2:N-r
    gamma = Y(1,1);
    if abs(gamma)<1e-14
        disp('Warning: small pivot (<1e-14)')
    end
    X = Y(1,2:N-k+1);
    f = [Y(2:r,1); A(k+r,k)]/gamma;
    %Z = [Y(2:r+1,2:N-k+1); A(k+r,k+1:N-r)];
    Z = [Y(2:r,2:N-k+1); A(k+r,k+1:N)];
    Y = -f*X + Z;
    L(:,:,k) = [1, zeros(1,r); -f, eye(r)];
    R(k,k) = gamma;
    R(k,k+1:N) = X;
end

% Step 3 (compute L_{N-r+1})
% Y is now Y_{N-r}
%Z = [Y; A(N,N-r+1:N)];
%[lastL,lastR] = myLU(Z);
[lastL,lastR] = myLU(Y);
L(1:r,1:r,N-r+1) = lastL;
R(N-r+1:N,N-r+1:N) = lastR;


%%%%%% Check (optional) %%%%%
% % the factorization here is LA = R
% B = A;
% for k = 1:N-r
%     B(k:k+r,:) = L(1:r+1,1:r+1,k)*B(k:k+r,:);
% end
% B(N-r+1:N,:) = L(1:r,1:r,N-r+1)*B(N-r+1:N,:);
% disp(norm(R-B))
%%%%%% End check %%%%%

end