% Test for LU-based inversion algorithm
% for two-sided banded matrices.
%
% Here we test for relative errors on
% rank-structured part and for
% running time w.r.t. N (matrix size)
% with fixed bandwidth. Matrices are random.

% Define matrix A to be inverted
% (band matrix with half-bandwidth r
r = 5;
Ntest = 500:50:2500;
times = zeros(length(Ntest),1);
errors = zeros(length(Ntest),1);
conds = zeros(length(Ntest),1);

for hh = 1:length(Ntest)
    N = Ntest(hh);
    disp(N)
    A = tril(triu(rand(N),-r),r) + r*eye(N);
    
    tic
    % Compute Green generators for inv(A)
    [p,Plast,a,q] = QRbanded(A,r);
    times(hh) = toc;
    
    % Recover lower triangular part of computed inverse
    B = zeros(N);
    B(1,1:r) = p(1,:);
    for j=2:N-r
        x=p(j,:);
        for k=j-1:-1:2
            B(j,k+r)=x*q(:,k);
            x=x*a(:,:,k);
        end
        B(j,r+1)=x*q(:,1);
        B(j,1:r)=x*a(:,:,1);
    end
    y = Plast;
    for j = N-r:-1:1
        B(N-r+1:N,j+r) = y*q(:,j);
        y = y*a(:,:,j);
    end
    B(N-r+1:N,1:r) = y;
    
    % Compare with Ainv
    Ainv = inv(A);
    errors(hh) = (norm(tril(B,r-1)-tril(Ainv,r-1)))/norm(tril(Ainv,r-1));
    conds(hh) = cond(A);
end
figure(1)
plot(log10(Ntest),log10(times))
title("Running times")
figure(2)
title("Relative errors")
plot(Ntest,log10(errors),'r')
hold on
plot(Ntest,log10(eps*conds),'k')
legend("errors","eps x cond(A)")