% Comparison on running time
% for structured and classical inversion algorithms.
% Matrices are one-side banded.

r = 5;
Ntest = 500:50:2500;
nt = 10;  % number of trials for each matrix size
time_sLU = zeros(length(Ntest),nt);  % structured LU
time_sQR = zeros(length(Ntest),nt);  % structured QR
time_inv = zeros(length(Ntest),nt);  % MATLAB inv (LU-based) 
time_sparseinv = zeros(length(Ntest),nt);  % sparse inv
time_cQR = zeros(length(Ntest),nt);  % classical QR

for hh = 1:length(Ntest)
    N = Ntest(hh);
    disp(N)
    for tcount = 1:nt
    A = triu(rand(N),-r)+r*eye(N); 
    
    
    %%%%%%%%%%%%%%%%
    % Structured QR
    %%%%%%%%%%%%%%%%
    tic
    % Compute Green generators for inv(A)
    [p,Plast,a,q] = QRlower(A,r);
    time_sQR(hh,tcount) = toc;
       
    
    %%%%%%%%%%%%%%%%
    % Structured LU
    %%%%%%%%%%%%%%%%
    tic
    % Compute Green generators for inv(A)
    [p,Plast,a,q] = LUlower(A,r);
    time_sLU(hh,tcount) = toc;
     
    
    %%%%%%%%%%%%%%%%
    % MATLAB inv
    %%%%%%%%%%%%%%%%
    tic
    C = inv(A);
    time_inv(hh,tcount) = toc;
    
    %%%%%%%%%%%%%%%%%%%
    % MATLAB sparse inv
    %%%%%%%%%%%%%%%%%%%
    As = sparse(A);
    tic
    Cs = inv(As);
    time_sparseinv(hh,tcount) = toc;
    
    %%%%%%%%%%%%%%%%
    % Classical QR
    %%%%%%%%%%%%%%%%
    tic
    [Q,R] = qr(A);
    Cqr = R\Q';
    time_cQR(hh,tcount) = toc;
    
    end
end

av_time_sLU = sum(time_sLU,2)/nt;  % structured LU
av_time_sQR = sum(time_sQR,2)/nt;  % structured QR
av_time_inv = sum(time_inv,2)/nt;  % MATLAB inv (LU-based) 
av_time_sparseinv = sum(time_sparseinv,2)/nt;  % sparse inv
av_time_cQR = sum(time_cQR,2)/nt; % classical QR

figure(1)
hold on
plot(log10(Ntest),log10(av_time_sLU))
plot(log10(Ntest),log10(av_time_sQR))
plot(log10(Ntest),log10(av_time_inv))
plot(log10(Ntest),log10(av_time_sparseinv))
plot(log10(Ntest),log10(av_time_cQR))